"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const ramda_1 = __importDefault(require("ramda"));
const md5_1 = __importDefault(require("../lib/md5"));
function compile({ projectRoot, metroConfigAbsPath, bundleConfigs, }) {
    const promises = bundleConfigs.map(({ entryAbsPath, outputDir, getBundleName, platforms = ['ios', 'android'], }) => {
        return platforms.map((platform) => {
            const bundleName = getBundleName ? getBundleName(platform, entryAbsPath) : `${path_1.default.basename(entryAbsPath, '.js')}.${platform}.jsbundle`;
            const bundleAbsPath = path_1.default.resolve(outputDir, bundleName);
            const bundleProcess = child_process_1.spawn('node', [
                require.resolve('react-native/local-cli/cli.js', { paths: [projectRoot] }),
                'bundle',
                '--platform',
                platform,
                '--dev',
                'false',
                '--entry-file',
                entryAbsPath,
                '--bundle-output',
                bundleAbsPath,
                '--config',
                metroConfigAbsPath,
                '--assets-dest',
                // outputDir
                // ios打出assets, android可以公用
                platform === 'ios' ? outputDir : '',
                '--sourcemap-output',
                `${bundleAbsPath}.map`,
            ], {
                // 忽略输入, 把 stdout 输出到父进程的 stdout, 自己处理 stderr
                // stdio: ['ignore', 'inherit', 'pipe'],
                stdio: 'pipe',
                cwd: projectRoot,
            });
            return new Promise((resolve, reject) => {
                let errOut = [];
                bundleProcess.stderr.on('data', data => {
                    errOut = errOut.concat(data.toString().split('\n'));
                });
                bundleProcess.on('close', (code) => __awaiter(this, void 0, void 0, function* () {
                    if (code === 0) {
                        try {
                            const bundleMd5 = yield md5_1.default(bundleAbsPath);
                            resolve({
                                entryAbsPath,
                                bundleAbsPath,
                                bundleMd5,
                                platform,
                            });
                            return;
                        }
                        catch (err) {
                            reject(err);
                        }
                    }
                    reject(errOut.join('\n'));
                }));
                bundleProcess.on('error', (err) => {
                    reject(`打包 RN 入口文件的进程无法启动, ${err}`);
                });
            });
        });
    });
    return Promise.all(ramda_1.default.flatten(promises));
}
exports.default = compile;
