"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRegisterComponents = exports.getActualComponents = void 0;
const ramda_1 = __importDefault(require("ramda"));
function getActualComponents(entryFileContent) {
    const reg = /^\s*(AppRegistry.registerComponent|registerApp)\(['"](\S+)['"]/gm;
    const components = [];
    let match;
    while (match = reg.exec(entryFileContent)) {
        components.push(match[2]);
    }
    return components;
}
exports.getActualComponents = getActualComponents;
function getRegisterComponents({ entryFileContent, configComponents = [] }) {
    const actualComponents = getActualComponents(entryFileContent);
    if (actualComponents.length === 1) {
        return actualComponents;
    }
    if (actualComponents.length <= 0) {
        return [];
    }
    // case3: 有多个, 但是在配置文件中没有指定需要哪一个
    if (!configComponents || configComponents.length <= 0) {
        // message: `入口文件 ${entry} 注册了多个组件: ${actualComponents.join(';')}\n将使用最后注册的组件: ${R.last(actualComponents)}`,
        return [ramda_1.default.last(actualComponents)];
    }
    // 以下代码, 均在存在 configComponents, 多个 actualComponents 情况下运行
    // case4: 有多个, 在配置文件中声明了使用的入口组件名称
    if (actualComponents.includes(ramda_1.default.head(configComponents))) {
        return [ramda_1.default.head(configComponents)];
    }
    // case5: 有多个, 在配置文件中声明了使用的入口名称, 但是实际不存在
    if (!actualComponents.includes(ramda_1.default.head(configComponents))) {
        return [ramda_1.default.last(actualComponents)];
    }
    throw new Error('cant find any registerComponent');
}
exports.getRegisterComponents = getRegisterComponents;
