"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const SEPARATOR = ', ';
let verbose = false;
let disabled = false;
const formatMessages = (messages) => chalk_1.default.reset(messages.join(SEPARATOR));
const success = (...messages) => {
    if (!disabled) {
        console.log(`${chalk_1.default.green.bold('success')} ${formatMessages(messages)}`);
    }
};
const info = (...messages) => {
    if (!disabled) {
        console.log(`${chalk_1.default.cyan.bold('info')} ${formatMessages(messages)}`);
    }
};
const warn = (...messages) => {
    if (!disabled) {
        console.warn(`${chalk_1.default.yellow.bold('warn')} ${formatMessages(messages)}`);
    }
};
const error = (...messages) => {
    if (!disabled) {
        console.error(`${chalk_1.default.red.bold('error')} ${formatMessages(messages)}`);
    }
};
const debug = (...messages) => {
    if (verbose && !disabled) {
        console.log(`${chalk_1.default.gray.bold('debug')} ${formatMessages(messages)}`);
    }
};
const log = (...messages) => {
    if (!disabled) {
        console.log(`${formatMessages(messages)}`);
    }
};
const setVerbose = (level) => {
    verbose = level;
};
const isVerbose = () => verbose;
const disable = () => {
    disabled = true;
};
const enable = () => {
    disabled = false;
};
exports.default = {
    success,
    info,
    warn,
    error,
    debug,
    log,
    setVerbose,
    isVerbose,
    disable,
    enable,
};
