"use strict";
/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const filterPlatformAssetScales_1 = __importDefault(require("./filterPlatformAssetScales"));
const getAssetDestPathAndroid_1 = __importDefault(require("./getAssetDestPathAndroid"));
const getAssetDestPathIOS_1 = __importDefault(require("./getAssetDestPathIOS"));
const logger_1 = __importDefault(require("../lib/logger"));
function saveAssets(assets, platform, assetsDest) {
    if (!assetsDest) {
        logger_1.default.warn('Assets destination folder is not set, skipping...');
        return Promise.resolve();
    }
    const getAssetDestPath = platform === 'android' ? getAssetDestPathAndroid_1.default : getAssetDestPathIOS_1.default;
    const filesToCopy = Object.create(null); // Map src -> dest
    assets.forEach(asset => {
        const validScales = new Set(filterPlatformAssetScales_1.default(platform, asset.scales));
        asset.scales.forEach((scale, idx) => {
            if (!validScales.has(scale)) {
                return;
            }
            const src = asset.files[idx];
            const dest = path_1.default.join(assetsDest, getAssetDestPath(asset, scale));
            filesToCopy[src] = dest;
        });
    });
    return copyAll(filesToCopy);
}
function copyAll(filesToCopy) {
    const queue = Object.keys(filesToCopy);
    if (queue.length === 0) {
        return Promise.resolve();
    }
    logger_1.default.info(`Copying ${queue.length} asset files`);
    return new Promise((resolve, reject) => {
        const copyNext = (error) => {
            if (error) {
                reject(error);
                return;
            }
            if (queue.length === 0) {
                logger_1.default.info('Done copying assets');
                resolve();
            }
            else {
                // queue.length === 0 is checked in previous branch, so this is string
                const src = queue.shift();
                const dest = filesToCopy[src];
                copy(src, dest, copyNext);
            }
        };
        copyNext();
    });
}
function copy(src, dest, callback) {
    const destDir = path_1.default.dirname(dest);
    mkdirp_1.default(destDir, (err) => {
        if (err) {
            callback(err);
            return;
        }
        fs_1.default.createReadStream(src)
            .pipe(fs_1.default.createWriteStream(dest))
            .on('finish', callback);
    });
}
exports.default = saveAssets;
