"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createModuleIdHOF_1 = require("./createModuleIdHOF");
// @ts-ignore
const ext_sdk_cliutils_1 = require("@hyext/ext-sdk-cliutils");
function filterModuleHOF(options) {
    if (options.mode === 'dev') {
        // 开发模式下打包所有
        return () => true;
    }
    const { projectRoot, filterModules, filterModulesVersion, imageBaseURL, SDKAlias = {} } = options;
    const modulesVersionHelper = new ext_sdk_cliutils_1.ModulesHelper({
        rootPath: projectRoot,
    });
    const moduleIdFactory = createModuleIdHOF_1.createModuleIdHOF({ projectRoot, SDKAlias })();
    const filterModulesVersionCache = Array.from(filterModulesVersion).reduce((acc, cur) => {
        const sepPos = cur.lastIndexOf('@');
        return Object.assign(Object.assign({}, acc), { [cur.slice(0, sepPos)]: cur.slice(sepPos + 1) });
    }, {});
    return function processModuleFilter(module) {
        var _a, _b, _c;
        const modulePath = module['path'];
        {
            // 模块系统是副作用, 只能在 baseModule 中建立
            // 如果业务包又重新执行的话, 会导致基础包中定义的模块被重置而无法被找到
            // 一般来说, filterModules 会包含. 但是如果构建sdk用的metro版本和构建业务的版本不一样
            // 可能会引起id的变化, 所以这里进行额外的校验
            // filterModules 里的ID: node_modules_metro-runtime_src_polyfills_require // 0.67
            // node_modules_metro_src_lib_polyfills_require // 0.62
            const metroModuleSystem = [
                '/node_modules/metro-runtime/src/polyfills/require.js',
                '/node_modules/metro/src/lib/polyfills/require.js',
            ];
            if (metroModuleSystem.some(xs => modulePath.endsWith(xs))) {
                return false;
            }
        }
        const moduleId = moduleIdFactory(modulePath);
        if (filterModules.has(moduleId)) {
            if (!filterModulesVersionCache[moduleId]) {
                return false;
            }
            const version = modulesVersionHelper.getVersion(modulePath);
            if (filterModulesVersionCache[moduleId] !== `${version}`) {
                console.warn(`version dismatch! base@${filterModulesVersionCache[moduleId]} - busi@${version}, path: ${modulePath}\n`);
                // throw new Error(`module ${modulePath} version dismatch: ${moduleId}@${version}`)
                // process.exit(1)
            }
            return false;
        }
        /*
         * FIXME 更新0.62.2的hook
         */
        if (modulePath.indexOf('__prelude__') >= 0) {
            // const hook = Buffer.from(process.env.HYEXT_RN_HOOK || '', 'base64')
            // const hookText = hook.toString('ascii')
            // module.output[0]['data']['code'] = hookText
            return true;
        }
        let code = (_c = (_b = (_a = module === null || module === void 0 ? void 0 : module.output) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b['data']) === null || _c === void 0 ? void 0 : _c['code'];
        if (!code)
            return true;
        /*
         * FIXME 优化这段逻辑, 修复非压缩状态下替换失效的问题
         * 应该是放到 transform/babel 里比较合适
         */
        const nameMatch = code.match(/name[ ]*:[ ]*"(\S+?)"/);
        if (nameMatch && nameMatch.length >= 1) {
            const name = nameMatch[1];
            const typeMatch = code.match(/type[ ]*:[ ]*"(\S+?)"/);
            if (typeMatch && typeMatch.length >= 1) {
                const type = typeMatch[1];
                const relativeMatch = code.match(/httpServerLocation[ ]*:[ ]*"(\S+?)"/);
                if (relativeMatch && relativeMatch.length >= 1) {
                    const relativePath = relativeMatch[1];
                    //read image baseURI from process.env
                    const uri = imageBaseURL + relativePath + '/' + name + '.' + type;
                    code = code.replace(/\.exports=(\S+)}/, `.exports={uri:"${uri}"}}`);
                    module.output[0]['data']['code'] = code;
                }
            }
        }
        return true;
    };
}
exports.default = filterModuleHOF;
module.exports = filterModuleHOF;
