"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTransformerFile = void 0;
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const crypto_1 = __importDefault(require("crypto"));
const handlebars_1 = __importDefault(require("handlebars"));
// @ts-ignore untyped metro-cache
const metro_cache_1 = require("metro-cache");
// @ts-ignore
const metro_config_1 = require("metro-config");
// @ts-ignore
const defaults_1 = require("metro-config/src/defaults");
const cli_plugin_metro_1 = require("@react-native-community/cli-plugin-metro");
const createModuleIdHOF_1 = require("./createModuleIdHOF");
const resolveReactNativePath_1 = __importDefault(require("./resolveReactNativePath"));
const filterModuleHOF_1 = __importDefault(require("./filterModuleHOF"));
/*
 * SDKRole: 'streamer' | 'audience' 仅在线上打包使用:
 * 1. filterModuleHOF 用来过滤filterModules 里指定的包
 * 2. transform 用来替换sdk
 */
function getDefaultConfig(options) {
    const { projectRoot, imageBaseURL = '', filterModules = new Set(), filterModulesVersion = new Set(), SDKAlias = {}, mode = 'dev', extUuid = 'huya-ext-shared-uuid', cacheVersion = '1.0', port = 8081, } = options;
    // metro 默认值: metro-config/src/defaults/index.js
    const officialDefaultConfig = defaults_1.getDefaultValues(projectRoot);
    // 
    const officialCliConfig = cli_plugin_metro_1.getDefaultConfig({
        platforms: {},
        reactNativePath: resolveReactNativePath_1.default(projectRoot),
    });
    const officialConfig = metro_config_1.mergeConfig(officialDefaultConfig, officialCliConfig);
    const sepPos = __dirname.indexOf('/node_modules/');
    const topMostNodeModules = `${__dirname.slice(0, sepPos)}/node_modules`;
    const CACHE_PATH = path_1.default.join(process.env.NODE_PACK_CACHE_ROOT || os_1.default.tmpdir(), 'huya-ext-metro-cache', mode === 'dev'
        ? `${path_1.default.basename(projectRoot)}@${cacheVersion}`
        : `${extUuid}@${cacheVersion}`);
    const hyextConfig = {
        resolver: {
            sourceExts: [...officialConfig.resolver.sourceExts, 'hycss'],
            platforms: ['ios', 'android', 'native', 'web'],
        },
        serializer: {
            processModuleFilter: filterModuleHOF_1.default({ projectRoot, imageBaseURL, SDKAlias, filterModules, filterModulesVersion, mode }),
            createModuleIdFactory: mode === 'dev' ? createModuleIdHOF_1.defaultFactory : createModuleIdHOF_1.createModuleIdHOF({ projectRoot, SDKAlias, }),
        },
        server: {
            port,
            // 中间件的执行顺序: communityCLI => metroMiddleware.
            // 这个中间件中的逻辑, 会晚于 communityCLI 的中间件执行
            enhanceMiddleware: undefined,
        },
        transformer: {
            babelPlugins: [],
            enableBabelRCLookup: false,
            getTransformOptions: () => __awaiter(this, void 0, void 0, function* () {
                return ({
                    transform: {
                        asyncRequireModulePath: require.resolve('metro-runtime/src/modules/asyncRequire'),
                        babelTransformerPath: require.resolve('metro-react-native-babel-transformer'),
                        experimentalImportSupport: false,
                        /*
                         * 开发环境下, 这个值设置为true, 否则会遇到问题:
                         * https://github.com/facebook/react-native/issues/28801
                         */
                        inlineRequires: mode === 'dev',
                        unstable_disableES6Transforms: false,
                    },
                    preloadedModules: false,
                    ramGroups: [],
                });
            }),
            // 这个地址不要轻易改动, 否则开发环境加载不出图片
            publicPath: mode === 'dev' ? '' : '/hyext-assets',
        },
        extUuid,
        cacheStores: [
            new metro_cache_1.FileStore({
                // 使用系统的临时文件夹, 由操作系统决定何时清除
                root: CACHE_PATH,
            })
        ],
        hasteMapCacheDirectory: CACHE_PATH,
        // 确保不同的小程序能产生不同的缓存值
        cacheVersion: mode === 'dev'
            ? `${path_1.default.basename(projectRoot)}@${cacheVersion}`
            : `${extUuid}@${cacheVersion}`,
        projectRoot,
        watchFolders: [projectRoot, topMostNodeModules],
        onReady: () => { },
        enableRootBabelRC: false,
    };
    return metro_config_1.mergeConfig(officialDefaultConfig, officialCliConfig, hyextConfig);
}
exports.default = getDefaultConfig;
function generateTransformerFile(options) {
    const { projectRoot, extUuid = 'huya-ext-shared-uuid', cacheVersion = '1.0', babelPlugins = [], hyextTransformerConfig = {}, } = options;
    const identity = crypto_1.default
        .createHash('sha1')
        .update([extUuid || projectRoot, cacheVersion].join('$'))
        .digest('hex');
    const tempalteFile = path_1.default.join(__dirname, './babelTransformer.js.hbs');
    const destFile = path_1.default.join(__dirname, `${identity}.babelTransformer.js`);
    const template = handlebars_1.default.compile(fs_extra_1.default.readFileSync(tempalteFile, 'utf8'));
    fs_extra_1.default.writeFileSync(destFile, template({
        HANDLE_BARS_PLUGINS: JSON.stringify(babelPlugins),
        HANDLE_BARS_HYEXT_TRANSFORMER_CONFIG: JSON.stringify(hyextTransformerConfig),
    }), {
        encoding: 'utf8',
    });
    return destFile;
}
exports.generateTransformerFile = generateTransformerFile;
