"use strict";
/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore untyped metro
const matrix_1 = __importDefault(require("@hyext/matrix"));
// @ts-ignore
const metro_core_1 = require("metro-core");
// @ts-ignore
const metro_config_1 = require("metro-config");
const path_1 = __importDefault(require("path"));
// @ts-ignore
const cli_server_api_1 = require("@react-native-community/cli-server-api");
const watchMode_1 = __importDefault(require("./watchMode"));
const getDefaultConfig_1 = __importStar(require("../matrix-config/getDefaultConfig"));
const constants_1 = require("../constants");
const HyTerminalReporter_1 = __importDefault(require("../lib/HyTerminalReporter"));
const hyExtMiddleware_1 = require("./hyExtMiddleware");
function matrixServer(config) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        let reportEvent;
        const terminal = new metro_core_1.Terminal(process.stdout);
        const ReporterImpl = getReporterImpl(config.customLogReporterPath);
        const terminalReporter = new ReporterImpl(terminal);
        const reporter = {
            update(event) {
                terminalReporter.update(event);
                if (reportEvent) {
                    reportEvent(event);
                }
            },
        };
        const defaultConfig = getDefaultConfig_1.default({ projectRoot: config.projectRoot, mode: 'dev' });
        const babelTransformerPath = getDefaultConfig_1.generateTransformerFile({
            projectRoot: config.projectRoot,
            babelPlugins: (_b = (_a = config.metroConfig) === null || _a === void 0 ? void 0 : _a.transformer) === null || _b === void 0 ? void 0 : _b.babelPlugins,
            hyextTransformerConfig: (_d = (_c = config.metroConfig) === null || _c === void 0 ? void 0 : _c.transformer) === null || _d === void 0 ? void 0 : _d.hyextTransformerConfig,
        });
        const finalConfig = metro_config_1.mergeConfig(defaultConfig, Object.assign(Object.assign({ reporter }, (config.metroConfig || {})), { transformer: {
                babelTransformerPath,
            }, server: {
                port: config.port || 8081,
            } }));
        const { middleware, websocketEndpoints, messageSocketEndpoint, eventsSocketEndpoint, } = cli_server_api_1.createDevServerMiddleware({
            host: config.host || constants_1.DEFAULT_HOST,
            port: finalConfig.server.port,
            watchFolders: finalConfig.watchFolders,
        });
        middleware.use(cli_server_api_1.indexPageMiddleware);
        const customEnhanceMiddleware = finalConfig.server.enhanceMiddleware;
        finalConfig.server.enhanceMiddleware = (metroMiddleware, server) => {
            if (customEnhanceMiddleware) {
                metroMiddleware = customEnhanceMiddleware(metroMiddleware, server);
            }
            // 请保持 middleware 在 metroMiddleware 之前执行
            // 否则 metro 处理 /symbolicate 请求时会得到空的body
            // https://github.com/facebook/metro/issues/660
            return hyExtMiddleware_1.hyExtMiddleware.use(middleware).use(metroMiddleware);
        };
        const serverInstance = yield matrix_1.default.runServer(finalConfig, {
            host: config.host || constants_1.DEFAULT_HOST,
            hmrEnabled: true,
            websocketEndpoints,
            onReady: config.onReady,
            onError: config.onError,
        });
        reportEvent = eventsSocketEndpoint.reportEvent;
        if (config.interactive !== false) {
            watchMode_1.default(messageSocketEndpoint);
        }
        serverInstance.keepAliveTimeout = 30000;
        return serverInstance;
    });
}
exports.default = matrixServer;
function getReporterImpl(customLogReporterPath) {
    if (customLogReporterPath === undefined) {
        return HyTerminalReporter_1.default;
    }
    try {
        // First we let require resolve it, so we can require packages in node_modules
        // as expected. eg: require('my-package/reporter');
        return require(customLogReporterPath);
    }
    catch (e) {
        if (e.code !== 'MODULE_NOT_FOUND') {
            throw e;
        }
        // If that doesn't work, then we next try relative to the cwd, eg:
        // require('./reporter');
        return require(path_1.default.resolve(customLogReporterPath));
    }
}
