"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const readline_1 = __importDefault(require("readline"));
const hooksStdout_1 = __importDefault(require("./hooksStdout"));
const logger_1 = __importDefault(require("../lib/logger"));
const constants_1 = require("../constants");
function printWatchModeInstructions() {
    logger_1.default.log('\n\n按 "r" 键重新加载小程序.\n按 "d" 键打开小程序的开发菜单');
}
function enableWatchMode(messageSocket, printInstructions = true) {
    // We need to set this to true to catch key presses individually.
    // As a result we have to implement our own method for exiting
    // and other commands (e.g. ctrl+c & ctrl+z)
    if (!process.stdin.setRawMode) {
        logger_1.default.debug('Watch mode is not supported in this environment');
        return;
    }
    readline_1.default.emitKeypressEvents(process.stdin);
    process.stdin.setRawMode(true);
    // We have no way of knowing when the dependency graph is done loading
    // except by hooking into stdout itself. We want to print instructions
    // right after its done loading.
    const restore = hooksStdout_1.default((output) => {
        if (output.includes(constants_1.MATRIX_READY_MESSAGE)) {
            if (printInstructions) {
                printWatchModeInstructions();
            }
            restore();
        }
    });
    process.stdin.on('keypress', (_key, data) => {
        const { ctrl, name } = data;
        if (ctrl === true) {
            switch (name) {
                case 'c':
                    process.exit();
                    break;
                case 'z':
                    process.emit('SIGTSTP', 'SIGTSTP');
                    // process.emit('SIGTSTP' as 'disconnect');
                    break;
            }
        }
        else if (name === 'r') {
            messageSocket.broadcast('reload', null);
            logger_1.default.info('重新加载小程序...');
        }
        else if (name === 'd') {
            messageSocket.broadcast('devMenu', null);
            logger_1.default.info('打开小程序开发菜单...');
        }
    });
}
exports.default = enableWatchMode;
