import fs from 'fs'
import os from 'os'
import path from 'path'
import Handlebars from 'handlebars'

interface Options {
  projectRoot: string

  /*
   * 小程序打包时, 会把相对路径的图片引用改成对cdn图片的引用
   * 1. 如果打包时把图片保存到 outputDir, 那么 imageBaseURL 和 publicPath 相同
   * 1. 如果打包时把图片保存到 outputDir/another-folder/, 那么 imageBaseURL 和 publicPath/another-folder/ 相同
   *
   */
  imageBaseURL: string

  /* 不传时将在 projectRoot 目录下生成 */
  cfgOutputDir?: string
  useHyCSS?: boolean
}

interface ProdOptions extends Options {
  mode: 'production'

  /* baseModules 的绝对路径 */
  filterModulesPath: string
}

interface DevOptions extends Options {
  mode: 'dev'
  filterModulesPath?: string
}

const getRawPath = (filePath: string) => os.platform() === 'win32' ? filePath.replace(/\\/g, '\\\\') : filePath

export default function generateBasicConfig(options: ProdOptions | DevOptions): string {
  const { projectRoot, imageBaseURL, cfgOutputDir, useHyCSS = true, mode, filterModulesPath } = options
  const cfgName = 'metro-config.js'
  const cfgPath = path.resolve(cfgOutputDir || projectRoot, cfgName)

  const generateBasicConfig = () => {
    const template = fs.readFileSync(path.resolve(__dirname, cfgName)).toString()
    const context = {
      PROJECT_ROOT: getRawPath(projectRoot),
      IMAGE_BASE_URL: imageBaseURL,
      FILTER_MODULES_PATH: getRawPath(filterModulesPath || ''),
      MODE: mode,
      USE_HY_CSS: useHyCSS,
      CSS_TRANSFORMER_PATH: getRawPath(path.resolve(__dirname, '../matrix-config/cssTransformer.js')),
    }

    const configContent = Handlebars.compile(template)(context)

    fs.writeFileSync(cfgPath, configContent)
  }
  generateBasicConfig()

  return cfgPath
}

