import R from 'ramda'

export function getActualComponents(entryFileContent: string): string[] {
  const reg = /^\s*(AppRegistry.registerComponent|registerApp)\(['"](\S+)['"]/gm

  const components = []

  let match
  while (match = reg.exec(entryFileContent)) {
    components.push(match[2])
  }

  return components
}

export function getRegisterComponents({ entryFileContent, configComponents = [] }: {
  entryFileContent: string
  configComponents?: string[]
}): string[] {
  const actualComponents = getActualComponents(entryFileContent)

  if (actualComponents.length === 1) {
    return actualComponents
  }

  if (actualComponents.length <= 0) {
    return []
  }

  // case3: 有多个, 但是在配置文件中没有指定需要哪一个
  if (!configComponents || configComponents.length <= 0) {
    // message: `入口文件 ${entry} 注册了多个组件: ${actualComponents.join(';')}\n将使用最后注册的组件: ${R.last(actualComponents)}`,
    return [R.last(actualComponents) as string]
  }

  // 以下代码, 均在存在 configComponents, 多个 actualComponents 情况下运行

  // case4: 有多个, 在配置文件中声明了使用的入口组件名称
  if (actualComponents.includes(R.head(configComponents) as string)) {
    return [R.head(configComponents) as string]
  }

  // case5: 有多个, 在配置文件中声明了使用的入口名称, 但是实际不存在
  if (!actualComponents.includes(R.head(configComponents) as string)) {
    return [R.last(actualComponents) as string]
  }

  throw new Error('cant find any registerComponent')
}
