import path from 'path'

type BlackItem = RegExp | string

export default function blacklist(additionalBlacklist?: BlackItem[]) {
  // Don't forget to everything listed here to `package.json`
  // modulePathIgnorePatterns.
  const sharedBlacklist: BlackItem[] = [
    /node_modules\/react\/dist\/.*/,
    /website\/node_modules\/.*/,
    /heapCapture\/bundle\.js/,
    /.*\/__tests__\/.*/,
  ];

  const escapeRegExp = (pattern: BlackItem) => {
    if (Object.prototype.toString.call(pattern) === '[object RegExp]') {
      // @ts-ignore
      return pattern.source.replace(/\//g, path.sep);
    } else if (typeof pattern === 'string') {
      var escaped = pattern.replace(/[\-\[\]\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
      // convert the '/' into an escaped local file separator
      return escaped.replace(/\//g, '\\' + path.sep);
    } else {
      throw new Error('Unexpected blacklist pattern: ' + pattern);
    }
  }

  return new RegExp(
    '(' +
    (additionalBlacklist || [])
    .concat(sharedBlacklist)
    .map(escapeRegExp)
    .join('|') +
    ')$',
  );
}
